'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var kebabCase = require('lodash/kebabCase');
var path$1 = require('upath');
var parseArgs = require('fn-args');
var last = require('lodash/last');
var path = require('path');
var fs = require('fs');
var decorator = require('@midwayjs/decorator');
var flatten = require('lodash/flatten');
var isEqual = require('lodash/isEqual');
var uniq = require('lodash/uniq');
var util = require('util');
var tslib = require('tslib');
var defaultsDeep = require('lodash/defaultsDeep');
var url = require('url');
var isFunction = require('lodash/isFunction');
var noop = require('lodash/noop');
var pickBy = require('lodash/pickBy');
var core = require('@midwayjs/core');
require('axios');
var request_dist_midwayjsHooksCoreRequest = require('../request/dist/midwayjs-hooks-core-request.cjs.prod.js');

function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

var kebabCase__default = /*#__PURE__*/_interopDefault(kebabCase);
var path__default = /*#__PURE__*/_interopDefault(path$1);
var parseArgs__default = /*#__PURE__*/_interopDefault(parseArgs);
var last__default = /*#__PURE__*/_interopDefault(last);
var flatten__default = /*#__PURE__*/_interopDefault(flatten);
var isEqual__default = /*#__PURE__*/_interopDefault(isEqual);
var uniq__default = /*#__PURE__*/_interopDefault(uniq);
var defaultsDeep__default = /*#__PURE__*/_interopDefault(defaultsDeep);
var url__default = /*#__PURE__*/_interopDefault(url);
var isFunction__default = /*#__PURE__*/_interopDefault(isFunction);
var noop__default = /*#__PURE__*/_interopDefault(noop);
var pickBy__default = /*#__PURE__*/_interopDefault(pickBy);

function isDevelopment() {
  if (process.env.MIDWAY_TS_MODE === 'true' ||
  /* istanbul ignore next */
          "production" === 'test' ||
  /* istanbul ignore next */
          "production" === 'development') {
    return true;
  }
  /* istanbul ignore next */


  return false;
}
function useHooksMiddleware(fn) {
  return (...args) => {
    /**
     * @description Hooks middleware
     * @example const middleware = (next) => { const ctx = useContext() }
     */
    if (parseArgs__default['default'](fn).length === 1) {
      const next = last__default['default'](args);
      return fn(next);
    }

    return fn(...args);
  };
}
function formatCode(code) {
  try {
    const prettier = lazyRequire('prettier');
    return prettier.format(code, {
      semi: true,
      singleQuote: true,
      parser: 'babel'
    });
  } catch (e) {
    console.log('format code error', e);
    return code;
  }
}
function isPathInside(child, parent) {
  const relation = path.relative(parent, child);
  return Boolean(relation && relation !== '..' && !relation.startsWith(`..${path.sep}`) && relation !== path.resolve(child));
}
function lazyRequire(id) {
  return eval('require')(id);
}
function isExportDefault(name) {
  return name === 'default';
}

class FileRouter {
  constructor(options) {
    var _options$useSourceFil;

    this.root = void 0;
    this.projectConfig = void 0;
    this.useSourceFile = void 0;
    this.root = options.root;
    this.projectConfig = options.projectConfig;
    this.useSourceFile = (_options$useSourceFil = options.useSourceFile) !== null && _options$useSourceFil !== void 0 ? _options$useSourceFil : isDevelopment();
  }

  get source() {
    // src
    if (this.useSourceFile) {
      return path$1.join(this.root, this.projectConfig.source);
    } // dist


    return path$1.join(this.root, this.projectConfig.build.outDir);
  } // src/apis/lambda


  getApiDirectory(baseDir) {
    return path$1.join(this.source, baseDir);
  }

  getRoute(file) {
    const {
      routes
    } = this.projectConfig;
    return routes.find(route => {
      const dir = this.getApiDirectory(route.baseDir);
      return this.inside(file, dir);
    });
  }

  isApiFile(file) {
    const extensions = ['.ts', '.tsx', '.js', '.jsx', '.mjs'];

    if (!extensions.includes(path$1.extname(file))) {
      return false;
    }

    const testExt = ['.test.ts', '.test.tsx', '.test.js', '.test.jsx'];

    if (testExt.some(ext => file.endsWith(ext))) {
      return false;
    }

    const route = this.getRoute(file);
    return !!route;
  }

  getFunctionId(file, functionName, isExportDefault) {
    const relativePath = path$1.relative(this.source, file); // src/apis/lambda/index.ts -> apis-lambda-index

    const id = kebabCase__default['default'](path$1.removeExt(relativePath, path$1.extname(relativePath)));
    const name = [id, isExportDefault ? '' : `-${functionName}`].join('');
    return name.toLowerCase();
  }

  inside(child, parent) {
    return isPathInside(path$1.toUnix(child), path$1.toUnix(parent));
  }

}

const PRE_DEFINE_PROJECT_CONFIG = Symbol.for('PRE_DEFINE_PROJECT_CONFIG');
const SNAPSHOT_SYMBOL = Symbol.for('MIDWAY_HOOKS_SNAPSHOT');
const EXPORT_DEFAULT_FUNCTION_ALIAS = '$default';

class EventGateway {
  constructor(options) {
    this.container = void 0;
    this.router = void 0;
    this.defaultApiClient = '@midwayjs/hooks-miniprogram-client';
    this.router = new FileRouter(options);
    useApiClientMatcher(this.createEventClientMatcher());
  }

  is(route) {
    return !!(route !== null && route !== void 0 && route.event);
  }

  createEventClientMatcher() {
    var _this$router$projectC, _this$router$projectC2;

    const userConfigClient = (_this$router$projectC = this.router.projectConfig) === null || _this$router$projectC === void 0 ? void 0 : (_this$router$projectC2 = _this$router$projectC.request) === null || _this$router$projectC2 === void 0 ? void 0 : _this$router$projectC2.client;
    const client = userConfigClient === '@midwayjs/hooks/request' ? this.defaultApiClient : userConfigClient;
    return createApiClientMatcher().route({
      event: 'wechat-miniprogram'
    }, {
      client
    });
  }

  createApi(options) {
    const {
      functionId,
      fn,
      route
    } = options;
    createFunctionContainer({
      fn,
      functionId,

      parseArgs(ctx, event) {
        let args;

        switch (route.event) {
          case 'wechat-miniprogram':
            args = event === null || event === void 0 ? void 0 : event.args;
        }

        return args || [];
      },

      handlerDecorators: [decorator.ServerlessTrigger(decorator.ServerlessTriggerType.EVENT, {
        functionName: functionId
      })]
    });
  }

}

async function createEventApiClient(file, code, router) {
  const {
    init,
    parse
  } = lazyRequire('es-module-lexer');
  await init;
  const [, exports] = parse(code);

  if (exports.length === 0) {
    return null;
  }

  const {
    request: {
      client
    }
  } = router.projectConfig;
  const funcs = exports.map(name => {
    const isExportDefault = name === 'default';
    return {
      functionId: router.getFunctionId(file, name, isExportDefault),
      isExportDefault: isExportDefault,
      name: isExportDefault ? '$default' : name
    };
  });
  return formatCode(buildClient(client, funcs));
}

function buildClient(client, funcs) {
  const buildFunction = func => {
    return `
      export ${func.isExportDefault ? 'default' : ''} function ${func.name} (...args) {
        return request({
          functionId: '${func.functionId}',
          data: { args }
        })
      }
    `;
  };

  const functions = funcs.map(buildFunction).join('\n');
  return `
    import { request } from '${client}';
    ${functions}
  `;
}

class GatewayManager {
  static getInstance(root, projectConfig) {
    var _GatewayManager$insta;

    (_GatewayManager$insta = GatewayManager.instance) !== null && _GatewayManager$insta !== void 0 ? _GatewayManager$insta : GatewayManager.instance = new GatewayManager(root, projectConfig);
    return GatewayManager.instance;
  }

  constructor(root, projectConfig) {
    this.gateways = [];
    this.gateways = this.getBuiltInGateways(projectConfig).concat(projectConfig.gateway).map(Adapter => new Adapter({
      root,
      projectConfig
    }));
  }

  getBuiltInGateways(userConfig) {
    const builtinGateways = new Set();

    for (const route of userConfig.routes) {
      if (route.basePath) builtinGateways.add(HTTPGateway);
      if (route.event) builtinGateways.add(EventGateway);
    }

    return Array.from(builtinGateways);
  }

  getGatewayByRoute(route) {
    const gateway = this.gateways.find(gateway => gateway.is(route));

    if (!gateway) {
      throw new Error(`Can't find the correct gateway adapter. Current route: ${route}`);
    }

    return gateway;
  }

}
GatewayManager.instance = void 0;

const K_MATCHER_CALLBACK = Symbol('K_MATCHER_CALLBACK');

class ApiClientBuilder {
  constructor() {
    this.matchers = [];
  }

  use(matcher) {
    this.matchers.push(matcher);
  }

  async build(file, code, router) {
    if (!router.isApiFile(file)) {
      return;
    }

    const {
      init,
      parse
    } = lazyRequire('es-module-lexer');
    await init;
    const [, exports] = parse(code);
    const route = router.getRoute(file);
    const renderMetas = [];

    for (const callback of this.callbacks) {
      for (const functionName of exports) {
        const meta = {
          file,
          route,
          functionName: isExportDefault(functionName) ? EXPORT_DEFAULT_FUNCTION_ALIAS : functionName,
          functionId: router.getFunctionId(file, functionName, isExportDefault(functionName))
        };
        const buildMeta = callback(meta);
        if (!buildMeta) continue;
        renderMetas.push(Object.assign({}, meta, buildMeta));
      }
    }

    const client = this.buildApiClient(renderMetas);
    return formatCode(client);
  }

  buildApiClient(renderMetas) {
    const clients = uniq__default['default'](renderMetas.map(buildMeta => buildMeta.client));
    const imports = clients.map((client, index) => `import { request as request$${index} } from '${client}';`).join('\n');

    const buildFunction = meta => {
      const clientIndex = clients.findIndex(client => client === meta.client);
      return `
      export ${meta.functionName === EXPORT_DEFAULT_FUNCTION_ALIAS ? 'default' : ''} function ${meta.functionName} (...args) {
        const options = Object.assign({
          functionId: '${meta.functionId}',
          data: { args },
        }, ${JSON.stringify(meta.metadata || {}, null, 2)})

        return request$${clientIndex}(options)
      }
    `;
    };

    const functions = renderMetas.map(buildFunction).join('\n');
    return `
      ${imports}
      ${functions}
    `;
  }

  get callbacks() {
    return flatten__default['default'](this.matchers.map(matcher => matcher[K_MATCHER_CALLBACK]));
  }

}

ApiClientBuilder.instance = new ApiClientBuilder();
class ApiClientMatcher {
  constructor() {
    this[K_MATCHER_CALLBACK] = [];
  }

  route(route, buildMeta) {
    this[K_MATCHER_CALLBACK].push(meta => {
      const isMatch = Object.keys(route).every(key => isEqual__default['default'](route[key], meta.route[key]));

      if (isMatch) {
        return buildMeta;
      }
    });
    return this;
  }

  match(matcher) {
    validateFunction(matcher, 'matcher');
    this[K_MATCHER_CALLBACK].push(matcher);
    return this;
  }

}
function useApiClientMatcher(matcher) {
  return ApiClientBuilder.instance.use(matcher);
}
function createApiClientMatcher() {
  return new ApiClientMatcher();
}
function buildApiClient(file, code, router, initializeGatewayManager = true) {
  // Ensure that the gateway manager is initialized
  if (initializeGatewayManager) {
    GatewayManager.getInstance(router.root, router.projectConfig);
  }

  return ApiClientBuilder.instance.build(file, code, router);
}

const SINGLE_CONCURRENCY_MODE = Symbol.for('MIDWAY_HOOKS_SINGLE_CONCURRENCY_MODE');
const GlobalLocalStorage = {
  isSingleConcurrencyMode: () => !!globalThis[SINGLE_CONCURRENCY_MODE],

  getStore(key) {
    return globalThis[SINGLE_CONCURRENCY_MODE][key];
  },

  async run(ctx, callback) {
    globalThis[SINGLE_CONCURRENCY_MODE] = ctx;
    return await callback();
  }

};
const AsyncLocalStorage = {
  get runtime() {
    return lazyRequire('asynchronous-local-storage').als;
  },

  getStore(key) {
    return AsyncLocalStorage.runtime.get(key);
  },

  run(ctx, callback) {
    return new Promise(resolve => {
      AsyncLocalStorage.runtime.runWith(async () => {
        resolve(await callback());
      }, ctx);
    });
  }

};
/**
 * @private
 * private api, may change without notice.
 * Use asynchronous-local-storage due to serverless environment does not support node.js 12.17.0
 */

const als = {
  get runtime() {
    return GlobalLocalStorage.isSingleConcurrencyMode() ? GlobalLocalStorage : AsyncLocalStorage;
  },

  getStore(key) {
    return als.runtime.getStore(key);
  },

  run(ctx, callback) {
    return als.runtime.run(ctx, callback);
  }

};
function useContext() {
  return als.getStore('ctx');
}

//   constructor(actual: any, name: string, reason = 'is invalid') {
//     const message = `[ERR_INVALID_ARG_VALUE]: The argument '${name}' ${reason} ${getTypeMessage(
//       actual
//     )}`
//     super(message)
//   }
// }

class ERR_INVALID_ARG_TYPE extends Error {
  constructor(name, expected, actual) {
    const message = `[ERR_INVALID_ARG_TYPE]: The '${name}' argument must be of type ${expected}${getTypeMessage(actual)}`;
    super(message);
  }

} // fork from https://github.com/nodejs/node/blob/master/lib/internal/errors.js

function getTypeMessage(actual) {
  let msg = '';

  if (actual == null) {
    msg += `. Received ${actual}`;
  } else if (typeof actual === 'function' && actual.name) {
    msg += `. Received function ${actual.name}`;
  } else if (typeof actual === 'object') {
    if (actual.constructor && actual.constructor.name) {
      msg += `. Received an instance of ${actual.constructor.name}`;
    } else {
      const inspected = util.inspect(actual, {
        depth: -1
      });
      msg += `. Received ${inspected}`;
    }
  } else {
    let inspected = util.inspect(actual, {
      colors: false
    });
    if (inspected.length > 25) inspected = `${inspected.slice(0, 25)}...`;
    msg += `. Received type ${typeof actual} (${inspected})`;
  }

  return msg;
}

function validateArray(value, name) {
  if (!Array.isArray(value)) {
    throw new ERR_INVALID_ARG_TYPE(name, 'Array', value);
  }
}
function validateString(value, name) {
  if (typeof value !== 'string') throw new ERR_INVALID_ARG_TYPE(name, 'string', value);
} // export function validateOneOf(value: any, name: string, oneOf: any[]) {
//   if (!oneOf.includes(value)) {
//     const allowed = oneOf
//       .map((v) => (typeof v === 'string' ? `'${v}'` : String(v)))
//       .join(', ')
//     const reason = 'must be one of: ' + allowed
//     throw new ERR_INVALID_ARG_VALUE(name, value, reason)
//   }
// }

function validateFunction(value, name) {
  if (typeof value !== 'function') {
    throw new ERR_INVALID_ARG_TYPE(name, 'Function', value);
  }
}

function useLogger() {
  const ctx = useContext();
  return ctx.logger;
}
function usePlugin(key) {
  validateString(key, 'key');
  const ctx = useContext();
  return ctx.app[key] || ctx[key];
}
async function useInject(identifier) {
  if (typeof identifier !== 'function' && typeof identifier !== 'string') {
    throw new ERR_INVALID_ARG_TYPE('identifier', 'class, string', identifier);
  }

  const ctx = useContext();
  const requestContext = ctx['requestContext'];
  return requestContext.getAsync(identifier);
}
function useConfig(key) {
  if (key !== undefined) {
    validateString(key, 'key');
  }

  const ctx = useContext();
  const requestContext = ctx['requestContext'];
  return requestContext.getConfigService().getConfiguration(key);
}

function createSnapshot(snapshot) {
  globalThis[SNAPSHOT_SYMBOL] = snapshot;
}
function getSnapshot() {
  return globalThis[SNAPSHOT_SYMBOL];
}

function createFunctionContainer(options) {
  const {
    fn,
    functionId,
    parseArgs,
    handlerDecorators = [],
    classDecorators = [],
    runWithAsyncLocalStorage
  } = options;
  let FunctionContainer = class FunctionContainer {
    constructor() {
      this.ctx = void 0;
    }

    async handler(...handlerArgs) {
      const args = parseArgs(this.ctx, ...handlerArgs);
      return runWithAsyncLocalStorage ? await als.run({
        ctx: this.ctx
      }, async () => await fn(...args)) : await fn(...args);
    }

  };
  Object.defineProperty(FunctionContainer, 'name', {
    value: functionId
  });

  tslib.__decorate([decorator.Inject()], FunctionContainer.prototype, 'ctx', void 0);

  tslib.__decorate([...handlerDecorators], FunctionContainer.prototype, 'handler', null);

  FunctionContainer = tslib.__decorate([decorator.Provide(functionId), ...classDecorators], FunctionContainer);
  return FunctionContainer;
}

function getUrl(baseUrl, name) {
  if (name === EXPORT_DEFAULT_FUNCTION_ALIAS) {
    return baseUrl;
  }

  if (baseUrl.endsWith('/')) {
    return `${baseUrl}${name}`;
  }

  return `${baseUrl}/${name}`;
}

function createHTTPClientMatcher(router) {
  return createApiClientMatcher().match(meta => {
    var _router$projectConfig, _router$projectConfig2;

    const {
      file,
      route,
      functionName
    } = meta;

    if (!route.basePath) {
      return;
    }

    const baseUrl = router.getBaseUrl(file);
    const url = getUrl(baseUrl, functionName);
    const client = ((_router$projectConfig = router.projectConfig) === null || _router$projectConfig === void 0 ? void 0 : (_router$projectConfig2 = _router$projectConfig.request) === null || _router$projectConfig2 === void 0 ? void 0 : _router$projectConfig2.client) || '@midwayjs/hooks/request';
    return {
      client,
      metadata: {
        url,
        meta: {}
      }
    };
  });
}

function duplicateLogger(root, existPath, currentPath, api) {
  console.log('[%s] Duplicate routes detected. %s and %s both resolve to %s.', 'warn', path$1.relative(root, existPath), path$1.relative(root, currentPath), api);
}

class HTTPRouter extends FileRouter {
  constructor(...args) {
    super(...args);
    this.routes = new Map();
  }

  getBaseUrl(file) {
    const url = this.getHTTPPath(file, '', true);

    if (url === '/*') {
      return '/';
    }

    if (url.endsWith('/*')) {
      return url.slice(0, -2);
    }

    return url;
  }

  getHTTPPath(file, method, isExportDefault) {
    const {
      basePath,
      baseDir,
      underscore
    } = this.getRoute(file);
    const lambdaDirectory = this.getApiDirectory(baseDir);
    const {
      isCatchAllRoutes,
      filename
    } = this.parseFilename(path$1.basename(file, path$1.extname(file)));
    const fileRoute = filename === 'index' ? '' : filename; // TODO remove underscore support in future

    const func = isExportDefault ? '' : `${underscore ? '_' : ''}${method}`;
    const api = path$1.toUnix(path$1.join(basePath,
    /**
     * /apis/lambda/index.ts -> ''
     * /apis/lambda/note/index.ts -> 'note'
     */
    path$1.relative(lambdaDirectory, path$1.dirname(file)),
    /**
     * index -> ''
     * demo -> '/demo'
     */
    fileRoute, // underscore: getNoteList -> _getNoteList
    func, isCatchAllRoutes ? '/*' : ''));
    /**
     * duplicate routes
     */

    const existPath = this.routes.get(api);

    if (existPath && existPath !== path$1.toUnix(file)) {
      duplicateLogger(this.root, existPath, file, api);
    }

    this.routes.set(api, path$1.toUnix(file));
    return api;
  }

  parseFilename(filename) {
    var _re$exec;

    const re = /\[\.{3}(.+)]/;
    const isCatchAllRoutes = re.test(filename);
    return {
      isCatchAllRoutes,
      filename: isCatchAllRoutes ? (_re$exec = re.exec(filename)) === null || _re$exec === void 0 ? void 0 : _re$exec[1] : filename
    };
  }

}

class HTTPGateway {
  is(route) {
    return !!(route !== null && route !== void 0 && route.basePath);
  }

  constructor(options) {
    this.options = void 0;
    this.container = void 0;
    this.router = void 0;

    this.useAsyncLocalStorage = async (ctx, next) => {
      await als.run({
        ctx
      }, async () => await next());
    };

    this.options = options;
    this.router = new HTTPRouter(options);
    useApiClientMatcher(createHTTPClientMatcher(this.router));
  }

  createApi(options) {
    const {
      file,
      functionName,
      isExportDefault,
      fn
    } = options;
    const httpPath = this.router.getHTTPPath(file, functionName, isExportDefault);
    fn._param.url = httpPath;
    this.createHTTPApi(httpPath, options);
  }

  createHTTPApi(httpPath, options) {
    const {
      functionId,
      fn
    } = options; // setup middleware

    const middleware = [...fn.middleware];
    createFunctionContainer({
      runWithAsyncLocalStorage: false,
      fn,
      functionId,

      parseArgs(ctx) {
        var _ctx$request, _ctx$request$body;

        return ((_ctx$request = ctx.request) === null || _ctx$request === void 0 ? void 0 : (_ctx$request$body = _ctx$request.body) === null || _ctx$request$body === void 0 ? void 0 : _ctx$request$body.args) || [];
      },

      classDecorators: [decorator.Controller(httpPath)],
      handlerDecorators: [decorator.All('/', {
        middleware
      })]
    });
  }

  afterCreate() {
    if (isDevelopment()) {
      return;
    }

    if (!this.isViteProject) {
      return;
    }

    const {
      routes
    } = this.router;

    if (routes.has('/') || routes.has('/*')) {
      return;
    } // TODO get base dir


    const baseDir = 'TODO';
    const staticCache = lazyRequire('koa-static-cache');
    const mw = staticCache({
      dir: path$1.join(baseDir, '..', this.options.projectConfig.build.viteOutDir),
      dynamic: true,
      alias: {
        '/': 'index.html',

        /**
         * Add alias for windows, '/' -> '\\'
         * https://github.com/koajs/static-cache/blob/master/index.js#L45
         */
        '\\': 'index.html'
      },
      buffer: true,
      gzip: true
    });

    const fn = async () => {};

    fn.middleware = [mw];
    this.createHTTPApi('/*', {
      functionId: 'hooks-host',
      fn
    });
  }

  get isViteProject() {
    return ['vite.config.ts', 'vite.config.js'].some(config => fs.existsSync(path$1.join(this.options.root, config)));
  }

  async onReady(args) {
    var _runtimeConfig$middle, _runtimeConfig$middle2;

    const {
      app,
      runtimeConfig
    } = args;
    const mws = [this.useAsyncLocalStorage];
    (_runtimeConfig$middle = runtimeConfig.middleware) === null || _runtimeConfig$middle === void 0 ? void 0 : (_runtimeConfig$middle2 = _runtimeConfig$middle.forEach) === null || _runtimeConfig$middle2 === void 0 ? void 0 : _runtimeConfig$middle2.call(_runtimeConfig$middle, mw => mws.push(useHooksMiddleware(mw)));

    for (const mw of mws) {
      app.use(mw);
    }
  }

}

// Ignore Vite dev server
const VITE_REQUEST = ['@vite', '@react-refresh']; // Ref: https://github.com/vitejs/vite/blob/d156a9f364dddcfc41338f83e39db38a00a2ceb0/packages/vite/src/node/constants.ts#L49

const KNOWN_ASSET_TYPES = [// images
'png', 'jpe?g', 'gif', 'svg', 'ico', 'webp', 'avif', // media
'mp4', 'webm', 'ogg', 'mp3', 'wav', 'flac', 'aac', // fonts
'woff2?', 'eot', 'ttf', 'otf', // css
'css', 'less', 'sass', 'scss', // script
'm?jsx?', 'tsx?', 'json', 'map', // other
'wasm'];
const DEFAULT_ASSETS_RE = new RegExp(`\\.(` + KNOWN_ASSET_TYPES.join('|') + `)(\\?.*)?$`);
const ignorePattern = req => {
  if (VITE_REQUEST.some(api => req.url.includes(api))) {
    return true;
  }

  const {
    pathname,
    query
  } = url__default['default'].parse(req.url);
  return DEFAULT_ASSETS_RE.test(pathname) || DEFAULT_ASSETS_RE.test(query);
};

function getProjectRoot(cwd) {
  const {
    sync
  } = lazyRequire('pkg-dir');
  return sync(cwd) || process.cwd();
}
function setConfig(config) {
  globalThis[PRE_DEFINE_PROJECT_CONFIG] = config;
}
function getConfig(cwd) {
  const userConfig = globalThis[PRE_DEFINE_PROJECT_CONFIG] ? globalThis[PRE_DEFINE_PROJECT_CONFIG] : getConfigFromFile(cwd);
  return defaultsDeep__default['default']({}, userConfig, {
    source: './src/apis',
    dev: {
      ignorePattern
    },
    gateway: [],
    build: {
      viteOutDir: './build',
      outDir: './dist'
    },
    request: {
      client: '@midwayjs/hooks/request'
    }
  });
}

function getConfigFromFile(cwd) {
  const root = getProjectRoot(cwd);
  const configs = {
    ts: path__default['default'].join(root, 'midway.config.ts'),
    js: path__default['default'].join(root, 'midway.config.js')
  };

  if (!fs.existsSync(configs.ts) && !fs.existsSync(configs.js)) {
    throw new Error(`[ERR_INVALID_CONFIG] midway.config.ts is not found, root: ${root}`);
  }

  return fs.existsSync(configs.ts) ? requireByJiti(configs.ts) : requireByJiti(configs.js);
}

function defineConfig(config) {
  validateArray(config.routes, 'defineConfig.routes');
  return config;
}

const requireByJiti = id => {
  const createJITI = lazyRequire('jiti');
  const jiti = createJITI();
  const contents = jiti(id);
  if ('default' in contents) return contents.default;
  return contents;
};

class HooksComponent {
  constructor(options) {
    this.options = void 0;
    this.gatewayManager = void 0;
    this.options = options;
    this.gatewayManager = GatewayManager.getInstance(this.options.root, this.options.projectConfig);
  }

  load() {
    const snapshot = getSnapshot();

    if (snapshot) {
      this.loadBySnapshot(snapshot);
    } else {
      this.loadByScanner();
    }

    const configuration = core.createConfiguration({
      namespace: '@midwayjs/hooks'
    });
    configuration.onReady(async (container, app) => {
      for (const gateway of this.gateways) {
        var _gateway$onReady;

        await ((_gateway$onReady = gateway.onReady) === null || _gateway$onReady === void 0 ? void 0 : _gateway$onReady.call(gateway, {
          container,
          app,
          runtimeConfig: this.options.runtimeConfig
        }));
      }
    }).onStop(noop__default['default']);
    return {
      Configuration: configuration
    };
  }

  get gateways() {
    return this.gatewayManager.gateways;
  }

  loadByScanner() {
    const {
      sync
    } = lazyRequire('globby');
    const {
      router,
      projectConfig: {
        routes
      }
    } = this.options;

    for (const route of routes) {
      // Windows
      const dir = path__default['default'].join(router.source, route.baseDir);
      const files = sync([dir]).filter(file => router.isApiFile(file));

      for (const file of files) {
        const mod = require(file);

        this.loadApiModule({
          mod,
          file
        });
      }
    }

    this.gateways.forEach(gateway => {
      var _gateway$afterCreate;

      return (_gateway$afterCreate = gateway.afterCreate) === null || _gateway$afterCreate === void 0 ? void 0 : _gateway$afterCreate.call(gateway);
    });
  }

  loadBySnapshot(snapshot) {
    const {
      router
    } = this.options;
    const {
      container,
      modules
    } = snapshot; // Initialize container

    this.gateways.forEach(gateway => gateway.container = container); // Create api function

    for (const {
      mod,
      file
    } of modules) {
      if (router.isApiFile(file)) {
        this.loadApiModule({
          mod,
          file,
          container
        });
      }
    }

    this.gateways.forEach(adapter => {
      var _adapter$afterCreate;

      return (_adapter$afterCreate = adapter.afterCreate) === null || _adapter$afterCreate === void 0 ? void 0 : _adapter$afterCreate.call(adapter);
    });
  }

  loadApiModule({
    mod,
    file
  }) {
    const route = this.options.router.getRoute(file);
    const gateway = this.gatewayManager.getGatewayByRoute(route);
    this.createApi(mod, gateway, file, route);
  }

  createApi(mod, gateway, file, route) {
    var _mod$config;

    const modMiddleware = (mod === null || mod === void 0 ? void 0 : (_mod$config = mod.config) === null || _mod$config === void 0 ? void 0 : _mod$config.middleware) || [];
    const funcs = pickBy__default['default'](mod, isFunction__default['default']);

    for (const [name, fn] of Object.entries(funcs)) {
      fn.middleware = (fn.middleware || (fn.middleware = [])).concat(modMiddleware).map(useHooksMiddleware);
      const isExportDefault = name === 'default';
      const functionName = isExportDefault ? EXPORT_DEFAULT_FUNCTION_ALIAS : name;
      const functionId = this.options.router.getFunctionId(file, functionName, isExportDefault);
      fn._param = {
        functionId
      };
      gateway.createApi({
        fn,
        functionName,
        isExportDefault,
        functionId,
        file,
        route
      });
    }
  }

}

/**
 * @description Create hooks component
 */

const hooks = (runtimeConfig = {}) => {
  if (runtimeConfig.middleware !== undefined) {
    validateArray(runtimeConfig.middleware, 'runtimeConfig.middleware');
  }

  const options = {};
  const snapshot = getSnapshot();

  if (snapshot) {
    options.root = snapshot.root;
    options.projectConfig = snapshot.projectConfig;
  } else {
    options.root = getProjectRoot();
    options.projectConfig = getConfig();
  }

  const router = new FileRouter({
    root: options.root,
    projectConfig: options.projectConfig,
    useSourceFile: isDevelopment()
  });
  const hooks = new HooksComponent({
    root: options.root,
    router,
    runtimeConfig,
    projectConfig: options.projectConfig
  });
  return hooks.load();
};

exports.defaults = request_dist_midwayjsHooksCoreRequest.defaults;
exports.request = request_dist_midwayjsHooksCoreRequest.request;
exports.ApiClientMatcher = ApiClientMatcher;
exports.ERR_INVALID_ARG_TYPE = ERR_INVALID_ARG_TYPE;
exports.EXPORT_DEFAULT_FUNCTION_ALIAS = EXPORT_DEFAULT_FUNCTION_ALIAS;
exports.EventGateway = EventGateway;
exports.FileRouter = FileRouter;
exports.HTTPGateway = HTTPGateway;
exports.HTTPRouter = HTTPRouter;
exports.K_MATCHER_CALLBACK = K_MATCHER_CALLBACK;
exports.PRE_DEFINE_PROJECT_CONFIG = PRE_DEFINE_PROJECT_CONFIG;
exports.SINGLE_CONCURRENCY_MODE = SINGLE_CONCURRENCY_MODE;
exports.SNAPSHOT_SYMBOL = SNAPSHOT_SYMBOL;
exports.als = als;
exports.buildApiClient = buildApiClient;
exports.createApiClientMatcher = createApiClientMatcher;
exports.createEventApiClient = createEventApiClient;
exports.createFunctionContainer = createFunctionContainer;
exports.createHTTPClientMatcher = createHTTPClientMatcher;
exports.createSnapshot = createSnapshot;
exports.defineConfig = defineConfig;
exports.formatCode = formatCode;
exports.getConfig = getConfig;
exports.getProjectRoot = getProjectRoot;
exports.getSnapshot = getSnapshot;
exports.hooks = hooks;
exports.isDevelopment = isDevelopment;
exports.isExportDefault = isExportDefault;
exports.isPathInside = isPathInside;
exports.lazyRequire = lazyRequire;
exports.setConfig = setConfig;
exports.useApiClientMatcher = useApiClientMatcher;
exports.useConfig = useConfig;
exports.useContext = useContext;
exports.useHooksMiddleware = useHooksMiddleware;
exports.useInject = useInject;
exports.useLogger = useLogger;
exports.usePlugin = usePlugin;
exports.validateArray = validateArray;
exports.validateFunction = validateFunction;
exports.validateString = validateString;
